<?php
/**
 * @copyright	 Copyright © 2019 - All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 * @author      Planet Web GmbH
 * @author mail rene.kurt@planet-web.ch
 * @website     http://www.planet-web.ch
 *
 */
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class plgContentPWWeather extends JPlugin
{
	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
	}

	function onContentPrepare( $context, &$row, &$params, $page=0 )
	{
		$app = JFactory::getApplication();
      // Plugin code goes here.
      // You can access parameters via $this->params.
		
		// expression to search for
		$regex = "#{pwweather(.*?)}(.*?){/pwweather}#s";

		// find all instances of plugin and put in $matches
		$matches = array();
		preg_match_all( $regex, $row->text, $matches, PREG_SET_ORDER );
		$num = sizeof($matches);

		for ($i = 0; $i < $num; $i++) { 
			$weatherHTML = $this->generateWeather( $this->params, $matches[$i], $i );
			$row->text = preg_replace("{".$matches[$i][0]."}", $weatherHTML , $row->text, 1);
		}
		
		if($num > 0)
		{
			$currentURL = $this->params->def('websiteBaseURL');
			if ($currentURL == ''){
				if( isset($_SERVER['HTTPS'] ) ) {
					$currentURL = 'https://'.$_SERVER['HTTP_HOST'];
				}
				else {
					$currentURL = 'http://'.$_SERVER['HTTP_HOST'];
				}
 			}
			
			$currentURL = $currentURL . '/plugins/content/pwweather/';
		}
		
		return true;
	}
	
	private function generateWeather( $params, $options ) {
		$options[1] = str_replace (" ", "&", $options[1]); //replace space with &
		parse_str($options[1], $OptionArray);
		
		//weather attribute
		$urlstr = array();
		if(empty($OptionArray['lat']))
		{
			return "<strong>PW Weather Latitude is missing!</strong>";
		}
		else if(empty($OptionArray['lon']))
		{
			return "<strong>PW Weather Longitude is missing!</strong>";
		}
		
		
		if(empty($OptionArray['iconwidth']))
		{
			$OptionArray['iconwidth'] = "50";
		}
		
		if(!empty($OptionArray['enabled']) && $OptionArray['enabled'] === "false")
		{
			return str_replace ("enabled=false ", "", $options[0]); //remove this flag and config show string
		}

		$weatherText = "";
		date_default_timezone_set("Europe/Zurich");
		setlocale(LC_TIME, "de_CH.utf8");
		$jsonfile = file_get_contents("https://api.openweathermap.org/data/2.5/forecast?units=metric&lat=" . $OptionArray["lat"] . "&lon=" . $OptionArray["lon"] . "&appid=d0aa2681ebc094245b868b1b25388c8b");
		$jsonarray = json_decode($jsonfile, true);
		if ($params->def("title")) {
			$weatherText .= "<h2>Wetterdaten für " . $jsonarray["city"]["name"] . "</h2>";
		}
		$weatherText .= "<table border=\"0\">";
		// Header der Tabelle mit den Zeiten die wir bekommen, x Spalten anzeigen
		$weatherText .= "<tr><td></td>";
		$timearray = array("00:00:00", "03:00:00", "06:00:00", "09:00:00", "12:00:00", "15:00:00", "18:00:00", "21:00:00");
		for ($idx=0; $idx<sizeof($timearray); $idx++) {
			date_default_timezone_set("UTC");
			$dtstamp = strtotime($timearray[$idx]);
			date_default_timezone_set("Europe/Zurich");
			$width = intval($OptionArray["iconwidth"]) + 16;
			$weatherText .= "<td width=\"" . $width . "\" align=\"center\">" . strftime("%k:%M", $dtstamp) . "</td>";
		}
		$weatherText .= "</tr>";

		$firstday = substr($jsonarray["list"][0]["dt_txt"], 0, 10);
		for ($idx=0; $idx<$jsonarray["cnt"]; $idx++) {
			if ($tmpdate != substr($jsonarray["list"][$idx]["dt_txt"], 0, 10)) {
				if (! empty($tmpdate)) {
					$weatherText .= "</tr>";
				}
				// den Tag in die 1. Spalte schreiben
				date_default_timezone_set("UTC");
				$dtstamp = strtotime($jsonarray["list"][$idx]["dt_txt"]);
				date_default_timezone_set("Europe/Zurich");
				$weatherText .= "<tr><td>" . strftime("%A", $dtstamp) . "</td>";
				$tmpdate = substr($jsonarray["list"][$idx]["dt_txt"], 0, 10);
			}
			// 1. Tag muss speziell behandelt werden, da wir nicht alle Stunden bekommen
			if (substr($jsonarray["list"][$idx]["dt_txt"], 0, 10) == $firstday) {
				if ($idx == 0) {
					$pos = array_search(substr($jsonarray["list"][$idx]["dt_txt"], 11, 8), $timearray);
					for ($i=0; $i<$pos; $i++) {
						$weatherText .= "<td></td>";
					}
				}
			}
			if (substr($jsonarray["list"][$idx]["dt_txt"], 11, 8) >= $timearray[0] && substr($jsonarray["list"][$idx]["dt_txt"], 11, 8) <= $timearray[sizeof($timearray) - 1]) {
				$title = "";
				if ($params->def("temperature")) {
					$title .= number_format($jsonarray["list"][$idx]["main"]["temp"], 1, ".", "") . " °C";
				}
				if ($params->def("pressure")) {
					if (! empty($title)) {
						$title .= "\n";
					}
					$title .= number_format($jsonarray["list"][$idx]["main"]["pressure"], 0, ".", "") . " hpa";
				}
				if ($params->def("wind")) {
					if (! empty($title)) {
						$title .= "\n";
					}
					$title .= number_format($jsonarray["list"][$idx]["wind"]["speed"], 0, ".", "") . " m/s";
				}
				$weatherText .= "<td align=\"center\"><img src=\"https://openweathermap.org/img/w/" . $jsonarray["list"][$idx]["weather"]["0"]["icon"] . ".png\" width=\"" . $OptionArray["iconwidth"] . "\" title=\"" . $title . "\" /></td>";
			}
		}
		// Am letzten Tag noch die Zellen auffuellen wenn noetig
		$pos = array_search(substr($jsonarray["list"][39]["dt_txt"], 11, 8), $timearray);
		if ($pos === true) {
			for ($i=$pos; $i<sizeof($timearray) - 1; $i++) {
				$weatherText .= "<td></td>";
			}
		}
		$weatherText .= "</tr></table>";
		return $weatherText;
	}
}
?>
